local ActorInventory = require "engine.interface.ActorInventory"
local Stats = require "engine.interface.ActorStats"

damDesc = function(self, type, dam)
	-- Increases damage
	if self.inc_damage then
		local inc = (self.inc_damage.all or 0) + (self.inc_damage[type] or 0)
		dam = dam + (dam * inc / 100)
	end
	return dam
end

-- Feline talents
newTalentType{ type="race/feline", name = "feline", generic = true, description = "The various racial bonuses a character can have." }
racial_req1 = {
	level = function(level) return 0 + (level-1)  end,
}
racial_req2 = {
	level = function(level) return 4 + (level-1)  end,
}
racial_req3 = {
	level = function(level) return 8 + (level-1)  end,
}
racial_req4 = {
	level = function(level) return 12 + (level-1)  end,
}


newTalent{
	short_name = "SUDDEN_DASH",
	name = "Sudden Dash",
	image = "talents/celerity.png",
	type = {"race/feline", 1},
	require = racial_req1,
	points = 5,
	mode = "activated",
	no_energy = true,
	cooldown = function(self, t) 
		return math.ceil(self:combatTalentLimit(t, 10, 46, 30)) 
	end,  -- Limit to >10 turns
	getSpeed = function(self, t) return self:combatStatScale(math.max(self:getDex(), self:getWil()), 0.1, 0.476, 0.75) end,
	tactical = { DEFEND = 1 },
	
	action = function(self, t)
		self:setEffect(self.EFF_SPEED, 8, {power=t.getSpeed(self, t)})
		return true
	end,
	info = function(self, t)
		return ([[Call upon your instincts and inner reserves to increase your general speed by %d%% for 8 turns.
		The speed bonus will increase with your Dexterity or Willpower (whichever is higher).]]):
		format(t.getSpeed(self, t) * 100)
	end,
}

newTalent{
	name = "Feline Leap",
	image = "talents/duck_and_dodge.png",
	type = {"race/feline", 2},
	require = racial_req2,
	points = 5,
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 0, 19, 7)) end, -- Limit > 0
	mode = "activated",
	tactical = { CLOSEIN = 2, ESCAPE = 2 },
	range = function(self, t) return math.floor(self:combatTalentScale(t, 3, 7)) end,
	requires_target = true,
	direct_hit = true,
	no_energy = true,
	is_teleport = true,
	target = function(self, t)
		return {type="hit", range=self:getTalentRange(t), nolock=true, nowarning=true}
	end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		if not self:hasLOS(x, y) or game.level.map:checkEntity(x, y, Map.TERRAIN, "block_move") then
			game.logSeen(self, "You do not have line of sight.")
			return nil
		end
		
		local __, x, y = self:canProject(tg, x, y)

		game.level.map:particleEmitter(self.x, self.y, 1, "temporal_teleport")

		-- since we're using a precise teleport we'll look for a free grid first
		local tx, ty = util.findFreeGrid(x, y, 5, true, {[Map.ACTOR]=true})
		if tx and ty then
			if not self:teleportRandom(tx, ty, 0) then
				game.logSeen(self, "You fail to leap properly!")
			end
		end

		game.level.map:particleEmitter(self.x, self.y, 1, "temporal_teleport")
		game:playSoundNear(self, "talents/teleport")

		return true
	end,
	info = function(self, t)
		local range = self:getTalentRange(t)
		return ([[Teleports you to up to %d tiles away, to a targeted location in line of sight.  Additional talent points increase the range.
		This ability takes no time to use.]]):format(range)
	end,
}

newTalent{
	name = "Land on your Feet",
	image = "talents/earth_s_eyes.png",
	type = {"race/feline", 3},
	require = racial_req3,
	points = 5,
	mode = "passive",
	getChance = function(self, t) return self:combatStatLimit("lck", 1, .28, .745)*self:combatTalentLimit(t,100, 28,74.8) end, -- Limit < 100%
	do_unflinching_resolve = function(self, t)
		local effs = {}
		-- Go through all spell effects
		for eff_id, p in pairs(self.tmp) do
			local e = self.tempeffect_def[eff_id]
			if e.status == "detrimental" then
				if e.subtype.stun then 
					effs[#effs+1] = {"effect", eff_id}
				elseif e.subtype.blind and self:getTalentLevel(t) >=2 then
					effs[#effs+1] = {"effect", eff_id}
				elseif e.subtype.confusion and self:getTalentLevel(t) >=3 then
					effs[#effs+1] = {"effect", eff_id}
				elseif e.subtype.pin and self:getTalentLevel(t) >=4 then
					effs[#effs+1] = {"effect", eff_id}
				elseif e.subtype.slow and self:getTalentLevel(t) >=5 then
					effs[#effs+1] = {"effect", eff_id}
				end
			end
		end
		
		if #effs > 0 then
			local eff = rng.tableRemove(effs)
			if eff[1] == "effect" and rng.percent(t.getChance(self, t)) then
				self:removeEffect(eff[2])
				game.logSeen(self, "%s has recovered!", self.name:capitalize())
			end
		end
	end,
	info = function(self, t)
		local chance = t.getChance(self, t)
		return ([[Felines seem to recover from misfortune quickly. Each turn, you have a %d%% chance to recover from a single stun effect.
		At talent level 2 you may also recover from blindness, at level 3 confusion, level 4 pins, and level 5 slows. 
		Only one effect may be recovered from each turn, and the chance to recover from an effect scales with your Luck.]]):
		format(chance)
	end,
}

newTalent{
	short_name = "NINE_LIVES",
	name = "Nine Lives",
	image = "talents/halfling_luck.png",
	type = {"race/feline", 4},
	require = racial_req4,
	points = 5,
	mode="passive",
	getDiseaseImmune = function(self, t) return self:combatTalentLimit(t, 20, 5, 15) end, -- Limit < 100%
	passives = function(self, t, p)
		self:talentTemporaryValue(p, "inc_stats", { [self.STAT_LCK] = t.getDiseaseImmune(self, t) } )
	end,
	info = function(self, t)
		local power = t.getDiseaseImmune(self, t)
		return ([[You have tapped into the eternal wellspring of good fortune of your people.
		Your luck is enhanced by %.0f.]]):
		format(power)
	end,
}


-- Grey Canine Talents
newTalentType{ type="race/gcanine", name = "grey canine", generic = true, description = "The various racial bonuses a character can have." }

setupSummon = Talents.main_env.setupSummon

newTalent{
	short_name = "WOLF_ALLY",
	name = "Wolf Ally",
	image = "talents/war_hound.png",
	type = {"race/gcanine", 1},
	require = racial_req1,
	points = 5,
	random_ego = "attack",
	message = "@Source@ lets out a sudden howl, calling their wolf pack!",
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 0, 40, 10)) end, -- Limit > 0
	range = 0,
	requires_target = false,
	is_summon = true,
	tactical = { ATTACK = { PHYSICAL = 2 } },
	on_pre_use = function(self, t, silent)
		if not self:canBe("summon") and not silent then game.logPlayer(self, "You cannot summon; you are suppressed!") return end
		return true
	end,
	summonTime = function(self, t) return math.ceil(self:combatTalentLimit(t, 20, 5, 10)) end,
	incStats = function(self, t,fake)
		local mp = self:combatMindpower()
		return{ 
			str=15 + (fake and mp or self:mindCrit(mp)) * 2 * self:combatTalentScale(t, 0.2, 1, 0.75) + self:combatTalentScale(t, 2, 10, 0.75),
			dex=15 + (fake and mp or self:mindCrit(mp)) * 2 * self:combatTalentScale(t, 0.2, 1, 0.75) + self:combatTalentScale(t, 2, 10, 0.75),
			con=15
		}
	end,
	action = function(self, t)
		local tg = {type="bolt", nowarning=true, range=self:getTalentRange(t), nolock=true, talent=t}
		local tx=self.x
		local ty=self.y
		local target=self
		--local tx, ty, target = self:getTarget(tg)
		if not tx or not ty then return nil end
		local _ _, _, _, tx, ty = self:canProject(tg, tx, ty)
		target = game.level.map(tx, ty, Map.ACTOR)
		if target == self then target = nil end

		local z = 1
		if self:getTalentLevel(self.T_LEAD_THE_PACK) * 20 > math.random(1,100) then 
			z = z + 1
		end
		if self:getTalentLevel(self.T_LEAD_THE_PACK) * 5 > math.random(1,100) then 
			z = z + 1
		end
		for z=z,1,-1 do
			-- Find space
			local x, y = util.findFreeGrid(tx, ty, 5, true, {[Map.ACTOR]=true})
			if not x then
				game.logPlayer(self, "Not enough space to summon!")
				return
			end

			local NPC = require "mod.class.NPC"
			local m = NPC.new{
				type = "animal", subtype = "canine",
				display = "C", color=colors.LIGHT_DARK, image = "npc/canine_rungof.png",
				name = "pack member", faction = self.faction,
				desc = [[]],
				autolevel = "none",
				ai = "summoned", ai_real = "dumb_talented_simple", ai_state = { talent_in=5, },
				stats = {str=0, dex=0, con=0, cun=0, wil=0, mag=0},
				inc_stats = t.incStats(self, t),
				level_range = {self.level, self.level}, exp_worth = 0,
				global_speed_base = 1.2,

				max_life = resolvers.rngavg(25,50),
				life_rating = 6,
				infravision = 10,

				combat_armor = 2, combat_def = 4,
				combat = { dam=self:getTalentLevel(t) * 10 + rng.avg(12,25), atk=10, apr=10, dammod={str=0.8} },

				wild_gift_detonate = t.id,

				summoner = self, summoner_gain_exp=true, wild_gift_summon=true,
				summon_time = t.summonTime(self, t),
				ai_target = {actor=target}
			}
			setupSummon(self, m, x, y)
		end
		game:playSoundNear(self, "talents/spell_generic")
		return true
	end,
	info = function(self, t)
		local incStats = t.incStats(self, t, true)
		return ([[Summon a member of your pack for %d turns to attack your foes. Wolves are good basic melee attackers.
		It will get %d Strength, %d Dexterity and %d Constitution.
		Your summons inherit some of your stats: increased damage%%, stun/pin/confusion/blindness resistance, armour penetration.
		Their Strength and Dexterity will increase with your Mindpower.]])
		:format(t.summonTime(self, t), incStats.str, incStats.dex, incStats.con)
	end,
}

newTalent{
	name = "Lead the Pack",
	image = "talents/wayist.png",
	type = {"race/gcanine", 2},
	require = racial_req2,
	points = 5,
	mode = "passive",
	info = function(self, t)
		return ([[You prefer to lead a proper pack. 
		Gives you a %d%% chance at a second wolf and a %d%% chance at a third wolf when you summon your wolves.]]):
		format(20 * self:getTalentLevel(t),5 * self:getTalentLevel(t))
	end,
}

newTalent{
	name = "Wolf Howl",
	image = "talents/howl.png",
	type = {"race/gcanine", 3},
	require = racial_req3,
	points = 5,
	random_ego = "attack",
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 0, 30, 5)) end, -- Limit > 0
	range = 0,
	radius = function(self, t) return math.floor(self:combatTalentScale(t, 4, 8)) end,
	target = function(self, t)
		return {type="cone", range=self:getTalentRange(t), radius=self:getTalentRadius(t), selffire=false}
	end,
	requires_target = true,
	tactical = { DISABLE = 2 },
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		self:project(tg, x, y, function(px, py)
			local target = game.level.map(px, py, Map.ACTOR)
			if not target then return end
			target:setEffect(target.EFF_BATTLE_CRY, 7, {power=7 * self:getTalentLevel(t), apply_power=self:combatPhysicalpower()})
		end)
		game.level.map:particleEmitter(self.x, self.y, self:getTalentRadius(t), "directional_shout", {life=12, size=5, tx=x-self.x, ty=y-self.y, distorion_factor=0.1, radius=self:getTalentRadius(t), nb_circles=8, rm=0.8, rM=1, gm=0.8, gM=1, bm=0.1, bM=0.2, am=0.6, aM=0.8})
		game:playSoundNear(self, "creatures/wolves/wolf_howl_"..math.random(1,3))
		return true
	end,
	info = function(self, t)
		return ([[You unleash a haunting howl, striking terror in foes within a radius of %d, lowering their Defense by %d for 7 turns, making them easier to hit.
		The chance to hit increases with your Physical Power.]]):
		format(self:getTalentRadius(t), 7 * self:getTalentLevel(t))
	end,
}

newTalent{
	short_name = "WINTER_PELT",
	name = "Winter Pelt",
	image = "talents/icy_skin.png",
	type = {"race/gcanine", 4},
	require = racial_req4,
	points = 5,
	mode="passive",
	getDiseaseImmune = function(self, t) return self:combatTalentLimit(t, 30, 5, 20) end, -- Limit < 30
	passives = function(self, t, p)
		self:talentTemporaryValue(p, "resists", {[DamageType.FIRE] = (t.getDiseaseImmune(self, t) / 2) - t.getDiseaseImmune(self, t) / 5})
		self:talentTemporaryValue(p, "resists", { all = t.getDiseaseImmune(self, t) / 5})
		self:talentTemporaryValue(p, "resists", {[DamageType.DARKNESS] = (t.getDiseaseImmune(self, t)) - t.getDiseaseImmune(self, t) / 5})
		self:talentTemporaryValue(p, "resists", {[DamageType.COLD] = (t.getDiseaseImmune(self, t)) - t.getDiseaseImmune(self, t) / 5})
	end,
	info = function(self, t)
		local power = t.getDiseaseImmune(self, t)
		return ([[Your fur has become denser and carries the weight of your power, granting you resistance against cold and darkness of %d%% and %d%% against fire and %d%% against everything.]]):
		format(power,power / 2,power / 5)
	end,
}

-- Hooter Talents
newTalentType{ type="race/hooter", name = "furrae", generic = true, description = "The various racial bonuses a character can have." }

newTalent{
	short_name = "OBSERVE_THE_WORLD",
	name = "Keen Observation",
	image = "talents/keen_senses.png",
	type = {"race/hooter", 1},
	require = racial_req1,
	mode = "sustained",
	points = 5,
	--sustain_mana = 10,
	cooldown = 30,
	getSeeInvisible = function(self, t) return self:combatTalentScale(t, 1, 25) end,
	getSeeStealth = function(self, t) return self:combatTalentScale(t, 1, 10) end,
	getCriticalChance = function(self, t) return self:combatTalentScale(t, 1, 6) end,
	activate = function(self, t)
		game:playSoundNear(self, "talents/spell_generic")
		return {
			invis = self:addTemporaryValue("see_invisible", t.getSeeInvisible(self, t)),
			stealth = self:addTemporaryValue("see_stealth", t.getSeeStealth(self, t)),
			crit = self:addTemporaryValue("combat_spellcrit", t.getCriticalChance(self, t)),
		}
	end,
	deactivate = function(self, t, p)
		self:removeTemporaryValue("see_invisible", p.invis)
		self:removeTemporaryValue("see_stealth", p.stealth)
		self:removeTemporaryValue("combat_spellcrit", p.crit)
		return true
	end,
	info = function(self, t)
		local seeinvisible = t.getSeeInvisible(self, t)
		local seestealth = t.getSeeStealth(self, t)
		local criticalchance = t.getCriticalChance(self, t)
		return ([[Your keen eyes can see through the lies of the world, revealing the proper path.
		Improves your capacity to see invisible foes by +%d, to see through stealth by +%d, and to perform a critical spell cast by +%d%%.]]):
		format(seeinvisible, seestealth, criticalchance)
	end,
}

newTalent{
	name = "Clarity of Thought",
	image = "talents/highly_trained_mind.png",
	type = {"race/hooter", 2},
	require = racial_req2,
	points = 5,
	mode = "passive",
	stunImmune = function(self, t) return self:combatTalentLimit(t, 1, 0.17, 0.50) end,
	on_learn = function(self, t)
		self.combat_mentalresist = self.combat_mentalresist + 6
	end,
	on_unlearn = function(self, t)
		self.combat_mentalresist = self.combat_mentalresist - 6
	end,
	passives = function(self, t, p)
		self:talentTemporaryValue(p, "stun_immune", t.stunImmune(self, t))
	end,
	info = function(self, t)
		return ([[Improves Mental Saves by %d, and stun immunity by %d%%.]]):
		format(self:getTalentLevelRaw(t)*6, t.stunImmune(self, t)*100)
	end,
}

newTalent{
	name = "Sudden Flap",
	image = "talents/wing_buffet.png",
	type = {"race/hooter", 3},
	require = racial_req3,
	points = 5,
	random_ego = "defensive",
	tactical = { ESCAPE = 2, DEFEND = 2 },
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 10, 30, 15)) end, -- Limit > 10
	getDur = function(self, t) 
		return math.ceil(self:combatTalentLimit(t, 5, 2, 3)) 
	end, -- Limit > 5
	getChanceDef = function(self, t)
		--if self.perfect_evasion then return 100, 0 end
		return self:combatLimit(5*self:getTalentLevel(t) + (self.level * 2), 50, 10, 10, 37.5, 75),
		self:combatScale(self:getTalentLevel(t) * (self.level * 2), 0, 0, 55, 250, 0.75)
		-- Limit evasion chance < 50%, defense bonus ~= 55 at level 50
	end,
	action = function(self, t)
		local dur = t.getDur(self,t)
		local chance, def = t.getChanceDef(self,t)
		self:setEffect(self.EFF_EVASION, dur, {chance=chance, defense = def})
		return true
	end,
	info = function(self, t)
		local chance, def = t.getChanceDef(self,t)
		return ([[You take to the air, trying to outmaneuver your foes, granting you a %d%% chance to completely evade them and granting you %d defense for %d turns.
		]]):
		format(chance, def,t.getDur(self,t))
	end,
}

newTalent{
	short_name = "Haunting_Hoot",
	name = "Haunting Hoot",
	image = "talents/anomaly_gravity_spike.png",
	type = {"race/hooter", 4},
	require = racial_req4,
	points = 5,
	random_ego = "attack",
	message = "@Source@ hoots!",
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 0, 40, 15)) end, -- Limit > 0
	range = 0,
	radius = function(self, t) return math.floor(self:combatTalentScale(t, 3, 7)) end,
	target = function(self, t)
		return {type="ball", range=self:getTalentRange(t), radius=self:getTalentRadius(t), friendlyfire=false, talent=t}
	end,
	tactical = { DEFEND = 1, DISABLE = { confusion = 3 } },
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		self:project(tg, self.x, self.y, DamageType.CONFUSION, {
			dur=3,
			dam=0,
			power_check=function() return 20 end,
			resist_check=self.combatMentalResist,
		})
		game.level.map:particleEmitter(self.x, self.y, self:getTalentRadius(t), "shout", {additive=true, life=10, size=3, distorion_factor=0.5, radius=self:getTalentRadius(t), nb_circles=8, rm=0.8, rM=1, gm=0, gM=0, bm=0.1, bM=0.2, am=0.4, aM=0.6})
		return true
	end,
	info = function(self, t)
		local radius = self:getTalentRadius(t)
		return ([[You let out a haunting hoot that sends your foes into utter confusion for 3 turns in a radius of %d.
		]]):format(radius)
	end,
}

-- Blackback Talents
newTalentType{ type="race/blackback", name = "Blackback", generic = true, description = "The various racial bonuses a character can have." }

newTalent{
	short_name = "Musk_Spray",
	name = "Musk Spray",
	image = "talents/keen_senses.png",
	type = {"race/blackback", 1},
	require = racial_req1,
	points = 5,
	random_ego = "attack",
	message = "@Source@ lifts their tail and sprays musk!",
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 5, 15, 10)) end, -- Limit = 5
	tactical = { ATTACK = { ACID = 2 } },
	range = function(self, t) return math.floor(self:combatTalentScale(t, 5.5, 7.5)) end,
	direct_hit = function(self, t) if self:getTalentLevel(t) >= 5 then return true else return false end end,
	requires_target = true,
	target = function(self, t)
		local tg = {type="bolt", range=self:getTalentRange(t), talent=t}
		if self:getTalentLevel(t) >= 5 then tg.type = "beam" end
		return tg
	end,
	getDamage = function(self, t) 
		return self:combatTalentWeaponDamage(t, 15, 70) * ( ( 0.01 * self.level ) + 1  )
	end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		self:project(tg, x, y, DamageType.ACID_BLIND, self:mindCrit(t.getDamage(self, t)), nil)
		local _ _, x, y = self:canProject(tg, x, y)
		if tg.type == "beam" then
			game.level.map:particleEmitter(self.x, self.y, math.max(math.abs(x-self.x), math.abs(y-self.y)), "acidbeam", {tx=x-self.x, ty=y-self.y})
		else
			game.level.map:particleEmitter(x, y, 1, "acid")
		end
		game:playSoundNear(self, "talents/cloud")
		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self, t)
		return ([[Spray forth a glob of burning musk at your enemy.
		The target will take %0.2f acid damage.
		Enemies struck have a 25%% chance to be blinded for three turns, as they try to clear their eyes of burning goop.
		At Talent Level 5, this becomes a piercing line of acid.]]):format(damDesc(self, DamageType.ACID, damage))
	end,
}

newTalent{
	name = "Bee Raider",
	image = "talents/highly_trained_mind.png",
	type = {"race/blackback", 2},
	require = racial_req2,
	points = 5,
	mode = "passive",
	statusImmune = function(self, t) return self:combatTalentLimit(t, 1, 0.17, 0.50) end,
	
	passives = function(self, t, p)
		self:talentTemporaryValue(p, "poison_immune", t.statusImmune(self, t))
		self:talentTemporaryValue(p, "combat_physresist", self:getTalentLevelRaw(t)*6)
	end,
	info = function(self, t)
		return ([[Improves Physical Saves by %d, and poison immunity by %d%%.]]):
		format(self:getTalentLevelRaw(t)*6, t.statusImmune(self, t)*100)
	end,
}

newTalent{
	name = "Musky Aura",
	image = "talents/wing_buffet.png",
	type = {"race/blackback", 3},
	require = racial_req4,
	points = 5,
	mode = "sustained",
	--sustain_equilibrium = 10,
	remove_on_zero = false,
	cooldown = function(self, t) return 10 end,
	tactical = { ATTACKAREA = { ACID = 2 } },
	range = 0,
	radius = function(self, t)
		return math.ceil(self:combatTalentLimit(t, 5, 1, 2.5))
	end,
	target = function(self, t) 
		return {type="ball", range=0, radius=t.radius(self, t), selffire=false, friendlyfire=false} 
	end,
	requires_target = false,
	getNormalTarget = function(self, t)
		return {type="ball", range=0, radius=t.radius(self, t), selffire=false, friendlyfire=false}
	end,
	getAuraStrength = function(self, t)
		return self:combatTalentWeaponDamage(t, 5, 25) * ( ( 0.01 * self.level ) + 1 )
	end,
	callbackOnActBase = function(self, t)
		local tt = self:isTalentActive(t.id)
		if not tt then return end
		local dam = t.getAuraStrength(self, t)
		local tg = t.getNormalTarget(self, t)
		self:project(tg, self.x, self.y, DamageType.ACID_BLIND, t.getAuraStrength(self, t))
	end,
	activate = function(self, t)
		return {}
	end,
	deactivate = function(self, t, p)
		return true
	end,

	info = function(self, t)
		local dam = t.getAuraStrength(self, t)
		local radi = t.radius(self, t)
		return ([[Fills the air around you with your innate musk that deals %d acid damage to all who approach within %d squares of you, possibly blinding them.]]):
		format(damDesc(self, DamageType.ACID_BLIND, dam), radi)
	end,
}









-- Equis Talents
newTalentType{ type="race/equis", name = "Equis", generic = true, description = "The various racial bonuses a character can have." }

newTalent{
	name = "Buck",
	image = "talents/dirty_fighting.png",
	type = {"race/equis", 1},
	require = racial_req1,
	mode = "sustained",
	points = 5,
	cooldown = 12,
	no_energy = true,
	getDamageOnMeleeHit = function(self, t) return self:combatTalentPhysicalDamage(t, 5, 25) end,
	activate = function(self, t)
		game:playSoundNear(self, "talents/spell_generic2")
		local ret = {
			onhit = self:addTemporaryValue("on_melee_hit", {[DamageType.PHYSICAL]=t.getDamageOnMeleeHit(self, t)}),
		}
		return ret
	end,
	deactivate = function(self, t, p)
		self:removeTemporaryValue("on_melee_hit", p.onhit)
		return true
	end,
	info = function(self, t)
		local damageonmeleehit = t.getDamageOnMeleeHit(self, t)
		return ([[You prepare yourself, ready to lash out at aggressors. Any foolish enough to attack you in melee will be struck back at for %d physical damage.]]):
		format(damDesc(self, DamageType.PHYSICAL, damageonmeleehit))
	end,
}

newTalent{
	name = "Gallop",
	type = {"race/equis", 2},
	require = racial_req2,
	image = "talents/disengage.png",
	points = 5,
	random_ego = "attack",
	cooldown = function(self, t) return math.ceil(self:combatTalentLimit(t, 0, 36, 20)) end, --Limit to >0
	tactical = { ATTACK = { weapon = 1, stun = 1 }, CLOSEIN = 3 },
	requires_target = true,
	range = function(self, t) return math.floor(self:combatTalentScale(t, 6, 10)) end,
	on_pre_use = function(self, t)
		if self:attr("never_move") then return false end
		return true
	end,
	action = function(self, t)
		local tg = {type="hit", range=self:getTalentRange(t)}
		local x, y, target = self:getTarget(tg)
		if not x or not y or not target then return nil end
		if core.fov.distance(self.x, self.y, x, y) > self:getTalentRange(t) then return nil end

		local block_actor = function(_, bx, by) return game.level.map:checkEntity(bx, by, Map.TERRAIN, "block_move", self) end
		local linestep = self:lineFOV(x, y, block_actor)
		
		local tx, ty, lx, ly, is_corner_blocked 
		repeat  -- make sure each tile is passable
			tx, ty = lx, ly
			lx, ly, is_corner_blocked = linestep:step()
		until is_corner_blocked or not lx or not ly or game.level.map:checkAllEntities(lx, ly, "block_move", self)
		if not tx or core.fov.distance(self.x, self.y, tx, ty) < 1 then
			game.logPlayer(self, "You are too close to build up momentum!")
			return
		end
		if not tx or not ty or core.fov.distance(x, y, tx, ty) > 1 then return nil end

		local ox, oy = self.x, self.y
		self:move(tx, ty, true)
		if config.settings.tome.smooth_move > 0 then
			self:resetMoveAnim()
			self:setMoveAnim(ox, oy, 8, 5)
		end
		-- Attack ?
		if core.fov.distance(self.x, self.y, x, y) == 1 then

			if self:attackTarget(target, nil, 1.2, true) and target:canBe("stun") then
				-- Daze, no save
				target:setEffect(target.EFF_DAZED, 3, {})
			end
		end

		return true
	end,
	info = function(self, t)
		return ([[Rushes toward your target with incredible speed. If the target is reached, you get a free attack doing 120% weapon damage.
		If the attack hits, the target is dazed for 3 turns.
		You must rush from at least 2 tiles away.]])
	end,
}

newTalent{
	name = "Fast as a Horse",
	type = {"race/equis", 3},
	require = racial_req3,
	image = "talents/anomaly_haste.png",
	points = 5,
	mode = "passive",
	getSpeed = function(self, t) return self:combatTalentScale(t, 0.15, 0.5, 0.75) end,
	passives = function(self, t, p)
		self:talentTemporaryValue(p, "movement_speed", t.getSpeed(self, t))
	end,
	info = function(self, t)
		return ([[Calling upon your equine ancestry, you find yourself moving at greater speeds.
		Increases your movement speed by %d%%]]):
		format(t.getSpeed(self, t)*100)
	end,
}

newTalent{
	name = "Healthy as a Horse",
	type = {"race/equis", 4},
	require = racial_req4,
	image = "talents/no_fatigue.png",
	points = 5,
	mode = "passive",
	getDiseaseImmune = function(self, t) return self:combatTalentLimit(t, 20, 5, 15) end, -- Limit < 100%
	passives = function(self, t, p)
		self:talentTemporaryValue(p, "inc_stats", { [self.STAT_CON] = t.getDiseaseImmune(self, t) } )
	end,
	info = function(self, t)
		local power = t.getDiseaseImmune(self, t)
		return ([[You are, literally, as healthy as a horse.
		Your constitution is enhanced by %.0f.]]):
		format(power)
	end,
}
